package org.foxarmy.barcodescannerforemployees.views

import android.app.Activity
import android.content.Context
import android.graphics.Color
import android.graphics.drawable.GradientDrawable
import android.os.Build
import android.util.AttributeSet
import android.util.Log
import android.view.LayoutInflater
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.TextView
import androidx.annotation.RequiresApi
import androidx.constraintlayout.widget.ConstraintLayout
import androidx.core.content.ContextCompat
import androidx.core.graphics.blue
import androidx.core.graphics.green
import androidx.core.graphics.red
import org.foxarmy.barcodescannerforemployees.DBStorageController
import org.foxarmy.barcodescannerforemployees.R
import org.foxarmy.barcodescannerforemployees.dataclasses.AbstractProduct
import org.foxarmy.barcodescannerforemployees.dataclasses.Product
import org.foxarmy.barcodescannerforemployees.getActivity
import org.foxarmy.barcodescannerforemployees.getImageUri
import java.io.File
import java.time.LocalDate
import java.time.Period
import java.time.format.DateTimeFormatter

class ProductView: LinearLayout {
    var product: Product = Product()
    var isProductSelected = false
    private lateinit var activity: Activity

    private lateinit var productImageView: ImageView
    private lateinit var productNameTextView: TextView
    private lateinit var productNetWeightTextView: TextView
    private lateinit var productAmountTextView: TextView
    private lateinit var productCategoryView: TextView
    private lateinit var productLifeSpan: TextView

    constructor(context: Context, a: AttributeSet) : super(context, a) {
        activity = getActivity(context)!!
        val inflater: LayoutInflater = activity.layoutInflater
        inflater.inflate(R.layout.product_view, this)
    }

    @RequiresApi(Build.VERSION_CODES.O)
    constructor(activity: Activity, context: Context, product: Product) : super(context) {
        this.product = product
        this.activity = activity
        val inflater: LayoutInflater = activity.layoutInflater
        inflater.inflate(R.layout.product_view, this)

        productImageView = findViewById(R.id.productPicture)
        productNameTextView = findViewById(R.id.productNameView)
        productNetWeightTextView = findViewById(R.id.productNetWeightView)
        productAmountTextView = findViewById(R.id.amountView)
        productCategoryView = findViewById(R.id.categoryView)
        productLifeSpan = findViewById(R.id.dateSpan)

        findViewById<ConstraintLayout>(R.id.productLayout).setOnLongClickListener {
            isProductSelected = !isProductSelected
            updateStroke()
            true
        }

        update()
    }

    @RequiresApi(Build.VERSION_CODES.O)
    fun update () {
        val linkedAbstractProduct: AbstractProduct = DBStorageController(activity).findAbstractProductById(DBStorageController(activity).readableDatabase, product.abstractProductId)!!

        val picturesDir = File(activity.filesDir, "pictures")
        picturesDir.mkdirs()
        val pictureFile = File(picturesDir, "${linkedAbstractProduct.imageHash}.png")
        val imageUri = getImageUri(activity, pictureFile)
        productImageView.setImageURI(imageUri)

        productNameTextView.text = linkedAbstractProduct.name
        productNetWeightTextView.text = linkedAbstractProduct.netWeight.toString()
        productAmountTextView.text = product.amount.toString()
        productCategoryView.text = DBStorageController(activity).getCategoryNameById(DBStorageController(activity).readableDatabase, linkedAbstractProduct.category)
        productLifeSpan.text = "${product.dateOfProduction}-${product.dateOfExpiry}"

        updateStroke()
    }

    @RequiresApi(Build.VERSION_CODES.O)
    fun updateStroke() {
         if (isProductSelected) {
             this.background = ContextCompat.getDrawable(context, R.drawable.outline_selected)
        } else {
            this.background = ContextCompat.getDrawable(context, R.drawable.outline)
             val color = evaluateColor()
            (this.background.current as GradientDrawable).setStroke(3, color)
        }
    }

    @RequiresApi(Build.VERSION_CODES.O)
    fun evaluateColor(): Int {
        val dateFormatter: DateTimeFormatter =  DateTimeFormatter.ofPattern("d.M.yyyy")


        val fresh = LocalDate.parse(product.dateOfProduction, dateFormatter)
        val expired = LocalDate.parse(product.dateOfExpiry, dateFormatter)

        val shelfLife = Period.between(fresh, expired).days

        val today = LocalDate.parse(LocalDate.now().format(dateFormatter), dateFormatter)
        Log.d("QWERTYUIOP", today.toString())

        val daysBeforeExpiry = Period.between(today, expired).days

        val freshnessPercentage: Double = daysBeforeExpiry / shelfLife.toDouble()
        Log.d("QWERTYUIOP", "$daysBeforeExpiry, $shelfLife, $freshnessPercentage")

        return calculateFreshnessGradient(freshnessPercentage)
    }

    fun calculateFreshnessGradient(percentage: Double): Int {
        val startColor = ContextCompat.getColor(context, if (percentage > 0.5) R.color.full_freshness else R.color.half_freshness)
        val endColor = ContextCompat.getColor(context, if (percentage > 0.5) R.color.half_freshness else R.color.expired_freshness)

        val gradientPosition = 1 - if (percentage > 0.5) (percentage - 0.5) * 2 else percentage * 2

        Log.d("QWERTYUIOP", "GP: $gradientPosition")


        val red = (startColor.red + gradientPosition * (endColor.red - startColor.red )).toInt()
        val green = (startColor.green  + gradientPosition * (endColor.green - startColor.green)).toInt()
        val blue = (startColor.blue + gradientPosition * (endColor.blue - startColor.blue)).toInt()

        var redHex = java.lang.Integer.toHexString(red)
        if (redHex.length == 1) redHex = "0$redHex"

        var greenHex = java.lang.Integer.toHexString(green)
        if (greenHex.length == 1) greenHex = "0$greenHex"

        var blueHex = java.lang.Integer.toHexString(blue)
        if (blueHex.length == 1 ) blueHex = "0$blueHex"


        val colorString = "#$redHex$greenHex$blueHex"
        Log.d("QWERTYUIOP", "Color: $colorString\nRGB:$red:$green:$blue")
        return Color.parseColor(colorString)
    }
}